@@ -13,7 +13,7 @@
 
 use strict;
 use warnings;
-use lib '/usr/share/apt-cacher';
+use lib '/usr/share/apt-cacher/lib';
 
 use sigtrap qw(die normal-signals);
 use Cwd;
@@ -59,7 +59,7 @@ Usage: $0  [-c|cfg|conf <configfile>] [-n|nice] [-s|simulate]
  [-v|verbose] [-o|offline] [-f|force] [-r|recover] ][-d|db <command> [<arg>]]
 
 Options:
- -n	Renice to lowest priority and continue.
+ -n	Renice (and ionice, if possible) to lowest priority and continue.
  -s	Simulate and just show what would be done.
  -o	Don't update index files. Overrides offline_mode from configfile.
  -p	Try to update index files by patching.
@@ -74,6 +74,7 @@ Options:
 	delete <arg>	delete entries matching regexp
 	import		read new checksum data from Packages and Sources files in cache_dir
 	compact		clean and compact database
+	failcheck       remove locks left by failed process
         verify		verify the database
 EOM
     }
@@ -87,7 +88,7 @@ if ($sim_mode) {
 #############################################################################
 ### configuration ###########################################################
 # Include the library for the config file parser
-require('apt-cacher-lib.pl');
+require('apt-cacher.pl');
 # Read in the config file and set the necessary variables
 
 # $cfg needs to be global for setup_ownership
@@ -116,7 +117,15 @@ if (($cfg->{user} && $cfg->{user} !~ 'root' && !$> )
 # Output data as soon as we print it
 local $| = 1;
 
-setpriority 0, 0, 20 if $nice_mode;
+load_checksum(); # Will disable checksum if BerkeleyDB not available
+
+if ($nice_mode) {
+    printmsg("Nice mode\n");
+    setpriority 0, 0, 20;
+    use IO_Prio qw(:all);
+    ioprio_set(IOPRIO_WHO_PROCESS, $$,IOPRIO_PRIO_VALUE(IOPRIO_CLASS_IDLE, 0)) == 0 ||
+      printmsg("ioprio_set failed: $!\n");
+}
 
 sub printmsg {
     my @args = @_;
@@ -393,7 +402,7 @@ sub pdiff {
 	    while (<$tfh>) {
 		$z->print($_);
 	    }
-	    $z->flush();
+	    close($z);
 	    my $size = -s $destfile;
 	    my $datestring = HTTP::Date::time2str;
 	    my $response = HTTP::Response->new(200, 'OK', ['Date' => $datestring,
@@ -402,21 +411,20 @@ sub pdiff {
 							   'Last-Modified' => $datestring]);
 	    $response->header('Content-Encoding' => $encoding) if $encoding;;
 	    write_header("$destfile.header", $response);
+	    printmsg("Linking to $name\n");
 	    # Get global lock and copy
 	    set_global_lock('copy patched file');
-	    printmsg("Linking to $name\n");
 	    unlink $name,  "../headers/$name";
-	    link $destfile, $name or die "Link $destfile failed: $!";
+	    link $destfile, $name || die "Link $destfile failed: $!";
 	    link "$destfile.header", "../headers/$name" || die "Link $destfile.header failed: $!";
+	    release_global_lock();
 	    printmsg "Unlink temporary files\n";
 	    unlink $destfile, "$destfile.header";
-	    release_global_lock();
 	    # Read checksums
 	    if ($cfg->{checksum}) {
 		printmsg ("Importing new checksums from patched $name\n");
-		import_sums($z);
+		import_sums($name);
 	    }
-	    close($z);
 	}
     }
     else {
@@ -432,14 +440,14 @@ sub pdiff {
 sub db_compact {
     my ($dbh) = @_;
     printmsg "Compacting checksum database....\n";
-    while (my ($status, $results) = _db_compact($dbh)) {
-	if ($$status) {
-	    printmsg "db_compact failed: $$status\n";
+    while (my ($status, %results) = @{_db_compact($dbh)}) {
+	if ($status) {
+	    printmsg "db_compact failed: $status\n";
 	    last;
 	}
 	else {
-	    printmsg " Compacted ". $results->{compact_pages_free} ." pages\n Freed ". $results->{compact_pages_truncated} ." pages\n";
-	    if ($results->{compact_pages_free} + $results->{compact_pages_truncated} == 0) {
+	    printmsg " Compacted ". $results{compact_pages_free} ." pages\n Freed ". $results{compact_pages_truncated} ." pages\n";
+	    if ($results{compact_pages_free} + $results{compact_pages_truncated} == 0) {
 		printmsg "Done!\n";
 		last;
 	    }
@@ -456,8 +464,6 @@ if (@db_mode || $db_recover){
     $ok_chars .= '/' if $cfg->{distinct_namespaces};
     print "Checksum database mode\n";
 
-    load_checksum();
-
     if (!$cfg->{checksum}) {
 	die "$0: checksumming not enabled. Use --force to override\n" if !$force;
 	print "$0: checksumming not enabled, but forced to continue\n";
@@ -516,7 +522,15 @@ if (@db_mode || $db_recover){
 	      {
 		next if $filename !~ /$re/;
 		printmsg "Deleting data for $filename\n";
-		$cursor->c_del && warn "c_del failed: $BerkeleyDB::Error" if !$sim_mode;
+		$cursor->c_del == 0 || warn "c_del failed: $BerkeleyDB::Error" if !$sim_mode;
+	    }
+	    next SWITCH;
+	};
+	/^failcheck$/ && do {
+	    printmsg 'Connecting to database....';
+	    # Just connect to the database which runs failchk()
+	    if (db(1)) { # Without locking
+		printmsg "Success!\n";
 	    }
 	    next SWITCH;
 	};
@@ -666,8 +680,6 @@ printmsg "Building source package file/version table\n";
 my %svrhash = map {m#([-+.a-z0-9]+_(?:\d:)?[-+.~a-zA-Z0-9]+)\.dsc# && $1 => 1} keys %valid;
 
 # Remove old checksum data
-load_checksum(); # Will disable checksum if BerkeleyDB not available
-
 if ($cfg->{checksum}) {
 
     my $dbh = db();
@@ -682,7 +694,7 @@ if ($cfg->{checksum}) {
 	  {
 	      next if defined $valid{$filename};
 	      printmsg "Deleting checksum data for $filename\n";
-	      $cursor->c_del && warn "c_del failed: $BerkeleyDB::Error" if !$sim_mode;
+	      $cursor->c_del == 0 || warn "c_del failed: $BerkeleyDB::Error" if !$sim_mode;
 	      $do_compact = 1;
 	  }
 	db_compact($dbh) if $do_compact || $pdiff_mode;
@@ -698,7 +710,7 @@ foreach (glob('*{,/*}')) {
 	next;
     }
     else {
-	next unless is_package_file($_); # Package files only	
+	next unless is_file_type('package', $_); # Package files only	
     }
     if(! defined($valid{$_})) {
 	unlink $_, "../headers/$_", "../private/$_.complete" unless $sim_mode;
@@ -725,7 +737,7 @@ chdir "$cfg->{cache_dir}/headers" && -w "." || die "Could not enter the cache he
 
 foreach (glob('*{,/*}')) {
     next if -d; # Skip directories
-    if((is_package_file($_) && !defined($valid{$_})) # Not indexed
+    if((is_file_type('package', $_) && !defined($valid{$_})) # Not indexed
        || !-e "../packages/$_") { # No corresponding package
 	unlink $_, "../private/$_.complete" unless $sim_mode;
 	printmsg "Removing expired headers: $_ and company...\n";
