@@ -49,7 +49,7 @@ use strict;
 use warnings;
 
 # Include the library file
-use lib '/usr/share/apt-cacher';
+use lib '/usr/share/apt-cacher/lib';
 require('apt-cacher-lib.pl');
 
 use Getopt::Long qw(:config no_ignore_case bundling);
@@ -266,7 +266,7 @@ sub import {
     # Loop through all the package files in the current directory
     foreach my $packagefile (glob('*')) {
 	next unless -f $packagefile;
-	next unless $digest || is_package_file($packagefile);
+	next unless $digest || is_file_type('package', $packagefile);
 
 	# By default use the current name
 	my $targetfile = $packagefile;
