@@ -3,9 +3,9 @@
 use strict;
 use warnings;
 
-use lib '/usr/src/apt-cacher/src';
+use lib '/usr/src/apt-cacher/src/lib';
 
-require('apt-cacher-lib.pl');
+require('apt-cacher.pl');
 
 my $cfg = read_config('/dev/null'); # Get defaults with no config file
 
@@ -72,6 +72,13 @@ user = www-data
 #
 #checksum = 1
 
+# Importing checksums from new index files into the checksum database can cause
+# high CPU usage on slower systems. This option sets a limit to the number of
+# index files that are imported simultaneously, thereby limiting CPU load
+# average, but, possibly, taking longer. Set to 0 for no limit.
+#
+#concurrent_import_limit = 1
+
 # CGI mode is deprecated.
 #
 # Send a 410 (Gone) HTTP message with the specified text when accessed via
@@ -245,12 +252,18 @@ user = www-data
 # You shouldn't need to change anything below here. If you do, ensure you
 # understand the full implications of doing so.
 
-# Permitted package files - this is a perl regular expression which matches all
+# Permitted package files -- this is a perl regular expression which matches all
 # package-type files (files that are uniquely identified by their filename).
 #
 #package_files_regexp = $cfg->{package_files_regexp}
 
-# Permitted Index files - this is the perl regular expression which matches all
+# Permitted APT pdiff files -- this is a perl regular expression which matches
+# APT pdiff files which are ed(1) scripts used to patch index files rather than
+# redownloading the whole file afresh.
+#
+#pdiff_files_regexp = $cfg->{pdiff_files_regexp}
+
+# Permitted Index files -- this is the perl regular expression which matches all
 # index-type files (files that are uniquely identified by their full path and
 # need to be checked for freshness).
 #
