@@ -10,14 +10,14 @@
 use strict;
 use warnings;
 
-use lib '/usr/share/apt-cacher';
+use lib '/usr/share/apt-cacher/lib';
 
 umask 0022;
 
 #############################################################################
 ### configuration ###########################################################
 # Include the library for the config file parser
-require('apt-cacher-lib.pl');
+require('apt-cacher.pl');
 
 # Read in the config file and set the necessary variables
 our $cfg = eval{ read_config('/etc/apt-cacher/apt-cacher.conf') };
@@ -76,13 +76,17 @@ for ("README", "README.txt") {
    }
 }
 
-# Run database recovery
+# Run database recovery and reset semaphores
 if ($cfg->{checksum}) {
-    require('/usr/share/apt-cacher/apt-cacher-lib-cs.pl');
+    require('apt-cacher-cs.pl');
     setup_ownership();
     print "Running database recovery...";
     db_recover();
     print "Done!\n";
+    if (my $sem = get_existing_sem()) { # Existing only
+	print "Clearing semaphore block\n";
+	$sem->remove;
+    }
 }
 
 exit(0);
